@@ -30,6 +30,7 @@
 #include <stdbool.h>
 #include <stdint.h>
 #include <stdlib.h>
+#include <sys/types.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -50,7 +51,7 @@ enum econf_err {
   ECONF_NOGROUP = 4,
   /** Key not found */
   ECONF_NOKEY = 5,
-  /** Key has empty value */
+  /** Key is NULL or has empty value */
   ECONF_EMPTYKEY = 6,
   /** Error creating or writing to a file */
   ECONF_WRITEERROR = 7,
@@ -63,7 +64,25 @@ enum econf_err {
   /** Empty section name */
   ECONF_EMPTY_SECTION_NAME = 11,
   /** Text after section */
-  ECONF_TEXT_AFTER_SECTION = 12
+  ECONF_TEXT_AFTER_SECTION = 12,
+  /** Parsed file list is NULL */
+  ECONF_FILE_LIST_IS_NULL = 13,
+  /** Wrong boolean value (1/0 true/false yes/no) */
+  ECONF_WRONG_BOOLEAN_VALUE = 14,
+  /** Given key has NULL value */
+  ECONF_KEY_HAS_NULL_VALUE = 15,
+  /** File has wrong owner */
+  ECONF_WRONG_OWNER = 16,
+  /** File has wrong group */
+  ECONF_WRONG_GROUP = 17,
+  /** File has wrong file permissions */
+  ECONF_WRONG_FILE_PERMISSION = 18,
+  /** File has wrong dir permission */
+  ECONF_WRONG_DIR_PERMISSION = 19,
+  /** File is a sym link which is not permitted */
+  ECONF_ERROR_FILE_IS_SYM_LINK = 20,
+  /** User defined parsing callback has failed **/
+  ECONF_PARSING_CALLBACK_FAILED = 21
 };
 
 typedef enum econf_err econf_err;
@@ -134,8 +153,54 @@ typedef struct econf_file econf_file;
 extern econf_err econf_readFile(econf_file **result, const char *file_name,
 				    const char *delim, const char *comment);
 
+
+/** @brief Process the file of the given file_name and save its contents into key_file object.
+ *  The user defined function will be called in order e.g. to check the correct file permissions.
+ *
+ * @param result content of parsed file
+ * @param file_name absolute path of parsed file
+ * @param delim delimiters of key/value e.g. "\t =".
+ *        If delim contains space characters AND none space characters,
+ *        multiline values are not parseable.
+ * @param comment array of characters which define the start of a comment
+ * @param callback function which will be called for the given filename. This user defined function has
+ *        the pathname as paramter and returns true if this file can be parsed. If not, the
+ *        parsing will be aborted and ECONF_PARSING_CALLBACK_FAILED will be returned.
+ * @param callback_data pointer which will be given to the callback function.
+ * @return econf_err ECONF_SUCCESS or error code
+ *
+ * Usage:
+ * @code
+ *   #include "libeconf.h"
+ *
+ *   bool checkFile(const char *filename, const void *data) {
+ *      - checking code which returns true or false -
+ *	return true;
+ *   }
+ *
+ *   econf_file *key_file = NULL;
+ *   econf_err error;
+ *
+ *   error = econf_readFileWithCallback (&key_file, "/etc/test.conf", "=", "#", checkFile, NULL);
+ *
+ *   econf_free (key_file);
+ * @endcode
+ *
+ * Default behaviour if entries have the same name in one file: The
+ * first hit will be returned. Further entries will be ignored.
+ * This can be changed by setting the environment variable
+ * ECONF_JOIN_SAME_ENTRIES. In that case entries with the same name
+ * will be joined to one single entry.
+ */
+extern econf_err econf_readFileWithCallback(econf_file **result, const char *file_name,
+					    const char *delim, const char *comment,
+					    bool (*callback)(const char *filename, const void *data),
+					    const void *callback_data);
+
 /** @brief Merge the contents of two key_files objects. Entries in etc_file will be
  *         prefered.
+ *         Comment and delimiter tag will be taken from usr_file. This can be changed
+ *         by calling the functions econf_set_comment_tag and econf_set_delimiter_tag.
  *
  * @param merged_file merged data
  * @param usr_file First data block which has to be merged.
@@ -161,7 +226,7 @@ extern econf_err econf_readFile(econf_file **result, const char *file_name,
  *
  */
 extern econf_err econf_mergeFiles(econf_file **merged_file,
-				       econf_file *usr_file, econf_file *etc_file);
+				  econf_file *usr_file, econf_file *etc_file);
 
 /** @brief Evaluating key/values of a given configuration by reading and merging all
  *         needed/available files in two different directories (normally in /usr/etc and /etc).
@@ -196,12 +261,68 @@ extern econf_err econf_mergeFiles(econf_file **merged_file,
  *
  */
 extern econf_err econf_readDirs(econf_file **key_file,
-					  const char *usr_conf_dir,
-					  const char *etc_conf_dir,
-					  const char *project_name,
-					  const char *config_suffix,
-					  const char *delim,
-					  const char *comment);
+				const char *usr_conf_dir,
+				const char *etc_conf_dir,
+				const char *project_name,
+				const char *config_suffix,
+				const char *delim,
+				const char *comment);
+
+/** @brief Evaluating key/values for every given configuration files in two different
+ *  directories (normally in /usr/etc and /etc). For each parsed file the user defined function
+ *  will be called in order e.g. to check the correct file permissions.
+ *
+ * @param key_files list of parsed file(s).
+ *        Each entry includes all key/value, path, comments,... information of the regarding file.
+ * @param size Size of the evaluated key_files list.
+ * @param usr_conf_dir absolute path of the first directory (normally "/usr/etc")
+ * @param etc_conf_dir absolute path of the second directory (normally "/etc")
+ * @param project_name basename of the configuration file
+ * @param config_suffix suffix of the configuration file. Can also be NULL.
+ * @param delim delimiters of key/value e.g. "\t ="
+ *        If delim contains space characters AND none space characters,
+ *        multiline values are not parseable.
+ * @param comment array of characters which define the start of a comment
+ * @param callback function which will be called for each file. This user defined function has the
+ *        pathname as paramter and returns true if this file can be parsed. If not, the parsing of
+ *        all files will be aborted and ECONF_PARSING_CALLBACK_FAILED will be returned.
+ * @param callback_data pointer which will be given to the callback function.
+ * @return econf_err ECONF_SUCCESS or error code
+ *
+ * Example: Reading content of example.conf in /usr/etc and /etc directory.
+ * @code
+ *   #include "libeconf.h"
+ *
+ *   bool checkFile(const char *filename, const void *data) {
+ *      - checking code which returns true or false -
+ *	return true;
+ *   }
+ *
+ *   econf_file *key_file = NULL;
+ *   econf_err error;
+ *
+ *   error = econf_readDirsWithCallback (&key_file,
+ *                                       "/usr/etc",
+ *                                       "/etc",
+ *                                       "example",
+ *                                       "conf",
+ *                                       "=", "#",
+ *                                       checkFile,
+ *                                       NULL);
+ *
+ *   econf_free (key_file);
+ * @endcode
+ *
+ */
+ extern econf_err econf_readDirsWithCallback(econf_file **key_file,
+					     const char *usr_conf_dir,
+					     const char *etc_conf_dir,
+					     const char *project_name,
+					     const char *config_suffix,
+					     const char *delim,
+					     const char *comment,
+					     bool (*callback)(const char *filename, const void *data),
+					     const void *callback_data);
 
 /** @brief Evaluating key/values for every given configuration files in two different
  *  directories (normally in /usr/etc and /etc). Returns a list of read configuration
@@ -230,6 +351,40 @@ extern econf_err econf_readDirsHistory(econf_file ***key_files,
 				       const char *delim,
 				       const char *comment);
 
+/** @brief Evaluating key/values for every given configuration files in two different
+ *  directories (normally in /usr/etc and /etc). For each parsed file the user defined function
+ *  will be called in order e.g. to check the correct file permissions.
+ *  Returns a list of read configuration files and their values.
+ *
+ * @param key_files list of parsed file(s).
+ *        Each entry includes all key/value, path, comments,... information of the regarding file.
+ * @param size Size of the evaluated key_files list.
+ * @param usr_conf_dir absolute path of the first directory (normally "/usr/etc")
+ * @param etc_conf_dir absolute path of the second directory (normally "/etc")
+ * @param project_name basename of the configuration file
+ * @param config_suffix suffix of the configuration file. Can also be NULL.
+ * @param delim delimiters of key/value e.g. "\t ="
+ *        If delim contains space characters AND none space characters,
+ *        multiline values are not parseable.
+ * @param comment array of characters which define the start of a comment
+ * @param callback function which will be called for each file. This user defined function has the
+ *        pathname as paramter and returns true if this file can be parsed. If not, the parsing of
+ *        all files will be aborted and ECONF_PARSING_CALLBACK_FAILED will be returned.
+ * @param callback_data pointer which will be given to the callback function.
+ * @return econf_err ECONF_SUCCESS or error code
+ *
+ */
+extern econf_err econf_readDirsHistoryWithCallback(econf_file ***key_files,
+						   size_t *size,
+						   const char *usr_conf_dir,
+						   const char *etc_conf_dir,
+						   const char *project_name,
+						   const char *config_suffix,
+						   const char *delim,
+						   const char *comment,
+						   bool (*callback)(const char *filename, const void *data),
+						   const void *callback_data);
+
 /* The API/ABI of the following three functions (econf_newKeyFile,
    econf_newIniFile and econf_writeFile) are not stable and will change */
 
@@ -257,6 +412,41 @@ extern econf_err econf_newKeyFile(econf_file **result, char delimiter, char comm
  */
 extern econf_err econf_newIniFile(econf_file **result);
 
+/** @brief Returns the comment character tag of the given econf_file object.
+ *         This tag will be taken while writing comments to file.
+ *
+ * @param key_file econf_file object.
+ * @return char comment character tag
+ *
+ */
+extern char econf_comment_tag(econf_file *key_file);
+
+/** @brief Returns the delimiter character of the given econf_file object.
+ *         This delimiter will be taken while writing the data to file.
+ *
+ * @param key_file econf_file object.
+ * @return char delimiter of key/value
+ *
+ */
+extern char econf_delimiter_tag(econf_file *key_file);
+
+/** @brief Set the comment character tag of the given econf_file object.
+ *         This tag will be taken while writing comments to file.
+ *
+ * @param key_file econf_file object.
+ * @param comment comment tag
+ *
+ */
+extern void econf_set_comment_tag(econf_file *key_file, const char comment);
+
+/** @brief Set the delimiter character of the given econf_file object.
+ *         This delimiter will be taken while writing the data to file.
+ *
+ * @param key_file econf_file object.
+ * @param delimiter delimiter of key/value
+ *
+ */
+extern void econf_set_delimiter_tag(econf_file *key_file, const char delimiter);
 
 /** @brief Write content of a econf_file struct to specified location.
  *
@@ -624,6 +814,66 @@ extern void econf_freeArray(char **array);
  */
 extern void econf_freeFile(econf_file *key_file);
 
+/** @brief All parsed files require this user permission.
+ *         DEPRECATED: Use the callback in econf_readFileWithCallback,
+ *         econf_readDirsWithCallback or econf_readDirsHistoryWithCallback
+ *         instead.
+ *
+ * @param owner User ID
+ * @return void
+ *
+ */
+extern void __attribute__ ((deprecated("use one of econf_read*WithCallback instead")))
+econf_requireOwner(uid_t owner);
+
+/** @brief All parsed files require this group permission.
+ *         DEPRECATED: Use the callback in econf_readFileWithCallback,
+ *         econf_readDirsWithCallback or econf_readDirsHistoryWithCallback
+ *         instead.
+ *
+ * @param group Group ID
+ * @return void
+ *
+ */
+extern void __attribute__ ((deprecated("use one of econf_read*WithCallback instead")))
+econf_requireGroup(gid_t group);
+
+/** @brief All parsed file have to have these file and directory permissions.
+ *         DEPRECATED: Use the callback in econf_readFileWithCallback,
+ *         econf_readDirsWithCallback or econf_readDirsHistoryWithCallback
+ *         instead.
+ *
+ * @param file_perms file permissions
+ * @param dir_perms dir permissions
+ * @return void
+ *
+ */
+extern void __attribute__ ((deprecated("use one of econf_read*WithCallback instead")))
+econf_requirePermissions(mode_t file_perms, mode_t dir_perms);
+
+/** @brief Allowing the parser to follow sym links (default: true).
+ *         DEPRECATED: Use the callback in econf_readFileWithCallback,
+ *         econf_readDirsWithCallback or econf_readDirsHistoryWithCallback
+ *         instead.
+ *
+ * @param allow allow to follow sym links.
+ * @return void
+ *
+ */
+extern void __attribute__ ((deprecated("use one of econf_read*WithCallback instead")))
+econf_followSymlinks(bool allow);
+
+/** @brief Reset all UID, GID, permissions,... restrictions for parsed files/dirs.
+ *         DEPRECATED: Use the callback in econf_readFileWithCallback,
+ *         econf_readDirsWithCallback or econf_readDirsHistoryWithCallback
+ *         instead.
+ *
+ * @return void
+ *
+ */
+extern void __attribute__ ((deprecated("use one of econf_read*WithCallback instead")))
+econf_reset_security_settings(void);
+
 #ifdef __cplusplus
 }
 #endif
